@@ -14,8 +14,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from concurrent import futures
-
 import contextlib
 import datetime
 import errno
@@ -24,8 +22,12 @@ import logging
 import os
 import shutil
 import threading
+import weakref
+
+from concurrent import futures
 
 import fasteners
+import jsonschema
 from oslo_utils import timeutils
 import six
 
@@ -37,16 +39,63 @@ from tooz import utils
 LOG = logging.getLogger(__name__)
 
 
+class _Barrier(object):
+    def __init__(self):
+        self.cond = threading.Condition()
+        self.owner = None
+
+
 @contextlib.contextmanager
 def _translate_failures():
     try:
         yield
-    except EnvironmentError as e:
+    except (EnvironmentError, jsonschema.ValidationError) as e:
         coordination.raise_with_cause(coordination.ToozError,
                                       utils.exception_message(e),
                                       cause=e)
 
 
+_SCHEMA_TYPES = {
+    'array': (list, tuple),
+    'string': six.string_types + (six.binary_type,),
+}
+_SCHEMAS = {
+    'group': {
+        "type": "object",
+        "properties": {
+            "group_id": {
+                "type": "string"
+            },
+        },
+        "required": [
+            "group_id",
+        ],
+        "additionalProperties": True,
+    },
+    'member': {
+        "type": "object",
+        "properties": {
+            "member_id": {
+                "type": "string"
+            },
+            # For now joined_on and capabilities are left out as
+            # capabilities can be nearly arbitrary and joined_on is a date
+            # type (that is a python custom type).
+        },
+        "required": [
+            "member_id",
+        ],
+        "additionalProperties": True,
+    },
+}
+
+
+def _validate(data, schema_key):
+    schema = _SCHEMAS[schema_key]
+    jsonschema.validate(data, schema, types=_SCHEMA_TYPES)
+    return data
+
+
 def _lock_me(lock):
 
     def wrapper(func):
@@ -64,54 +113,68 @@ def _lock_me(lock):
 class FileLock(locking.Lock):
     """A file based lock."""
 
-    def __init__(self, path):
+    def __init__(self, path, barrier, member_id):
         super(FileLock, self).__init__(path)
         self.acquired = False
         self._lock = fasteners.InterProcessLock(path)
-        self._cond = threading.Condition()
+        self._barrier = barrier
+        self._member_id = member_id
+
+    def is_still_owner(self):
+        return self.acquired
 
     def acquire(self, blocking=True):
-        timeout = None
-        if not isinstance(blocking, bool):
-            timeout = float(blocking)
-            blocking = True
+        blocking, timeout = utils.convert_blocking(blocking)
         watch = timeutils.StopWatch(duration=timeout)
         watch.start()
-        while True:
-            with self._cond:
-                if self.acquired and blocking:
-                    if watch.expired():
-                        return False
-                    # If in the same process wait until we can attempt to
-                    # acquire it (aka, another thread should release it before
-                    # we can try to get it).
-                    self._cond.wait(watch.leftover(return_none=True))
-                elif self.acquired and not blocking:
+
+        # Make the shared barrier ours first.
+        with self._barrier.cond:
+            while self._barrier.owner is not None:
+                if not blocking or watch.expired():
                     return False
-                else:
-                    # All the prior waits may have left less time to wait...
-                    timeout = watch.leftover(return_none=True)
-                    self.acquired = self._lock.acquire(blocking=blocking,
-                                                       timeout=timeout)
-                    return self.acquired
+                self._barrier.cond.wait(watch.leftover(return_none=True))
+            self._barrier.owner = (threading.current_thread().ident,
+                                   os.getpid(), self._member_id)
+
+        # Ok at this point we are now working in a thread safe manner,
+        # and now we can try to get the actual lock...
+        gotten = False
+        try:
+            gotten = self._lock.acquire(
+                blocking=blocking,
+                # Since the barrier waiting may have
+                # taken a long time, we have to use
+                # the leftover (and not the original).
+                timeout=watch.leftover(return_none=True))
+        finally:
+            # NOTE(harlowja): do this in a finally block to **ensure** that
+            # we release the barrier if something bad happens...
+            if not gotten:
+                # Release the barrier to let someone else have a go at it...
+                with self._barrier.cond:
+                    self._barrier.owner = None
+                    self._barrier.cond.notify_all()
+
+        self.acquired = gotten
+        return gotten
 
     def release(self):
-        with self._cond:
-            if self.acquired:
-                self._lock.release()
-                self.acquired = False
-                self._cond.notify_all()
-                return True
-            else:
-                return False
+        if not self.acquired:
+            return False
+        with self._barrier.cond:
+            self.acquired = False
+            self._barrier.owner = None
+            self._barrier.cond.notify_all()
+        return True
 
     def __del__(self):
         if self.acquired:
             LOG.warn("Unreleased lock %s garbage collected", self.name)
 
 
-class FileDriver(coordination.CoordinationDriver,
-                 coordination._RunWatchersMixin):
+class FileDriver(coordination._RunWatchersMixin,
+                 coordination.CoordinationDriver):
     """A file based driver.
 
     This driver uses files and directories (and associated file locks) to
@@ -119,38 +182,56 @@ class FileDriver(coordination.CoordinationDriver,
     missing some functionality but in the future these not implemented API(s)
     will be filled in.
 
-    NOTE(harlowja): it does **not** automatically delete members from
-    groups of processes that have died, manual cleanup will be needed
-    for those types of failures.
+    General recommendations/usage considerations:
+
+    - It does **not** automatically delete members from
+      groups of processes that have died, manual cleanup will be needed
+      for those types of failures.
+
+    - It is **not** distributed (or recommended to be used in those
+      situations, so the developer using this should really take that into
+      account when applying this driver in there app).
     """
 
     HASH_ROUTINE = 'sha1'
     """This routine is used to hash a member (or group) id into a filesystem
        safe name that can be used for member lookup and group joining."""
 
+    _barriers = weakref.WeakValueDictionary()
+    """
+    Barriers shared among all file driver locks, this is required
+    since interprocess locking is not thread aware, so we must add the
+    thread awareness on-top of it instead.
+    """
+
     def __init__(self, member_id, parsed_url, options):
         """Initialize the file driver."""
         super(FileDriver, self).__init__()
         self._member_id = member_id
         self._dir = parsed_url.path
+        self._executor = utils.ProxyExecutor.build("File", options)
         self._group_dir = os.path.join(self._dir, 'groups')
         self._driver_lock_path = os.path.join(self._dir, '.driver_lock')
-        self._driver_lock = locking.SharedWeakLockHelper(
-            self._dir, FileLock, self._driver_lock_path)
+        self._driver_lock = self._get_raw_lock(self._driver_lock_path,
+                                               self._member_id)
         self._reserved_dirs = [self._dir, self._group_dir]
         self._reserved_paths = list(self._reserved_dirs)
         self._reserved_paths.append(self._driver_lock_path)
-        self._executor = None
         self._joined_groups = set()
         self._safe_member_id = self._make_filesystem_safe(member_id)
 
+    @classmethod
+    def _get_raw_lock(cls, path, member_id):
+        lock_barrier = cls._barriers.setdefault(path, _Barrier())
+        return FileLock(path, lock_barrier, member_id)
+
     def get_lock(self, name):
         path = utils.safe_abs_path(self._dir, name.decode())
         if path in self._reserved_paths:
             raise ValueError("Unable to create a lock using"
                              " reserved path '%s' for lock"
                              " with name '%s'" % (path, name))
-        return locking.SharedWeakLockHelper(self._dir, FileLock, path)
+        return self._get_raw_lock(path, self._member_id)
 
     @classmethod
     def _make_filesystem_safe(cls, item):
@@ -159,25 +240,12 @@ class FileDriver(coordination.CoordinationDriver,
     def _start(self):
         for a_dir in self._reserved_dirs:
             utils.ensure_tree(a_dir)
-        self._executor = futures.ThreadPoolExecutor(max_workers=1)
+        self._executor.start()
 
     def _stop(self):
         while self._joined_groups:
             self.leave_group(self._joined_groups.pop())
-        if self._executor is not None:
-            self._executor.shutdown(wait=True)
-            self._executor = None
-
-    def _submit(self, cb, *args, **kwargs):
-        executor = self._executor
-        if executor is None:
-            raise coordination.ToozError("File driver has not been started")
-        else:
-            try:
-                return executor.submit(cb, *args, **kwargs)
-            except RuntimeError:
-                raise coordination.ToozError("File driver asynchronous"
-                                             " executor has been shutdown")
+        self._executor.stop()
 
     def create_group(self, group_id):
         safe_group_id = self._make_filesystem_safe(group_id)
@@ -197,7 +265,7 @@ class FileDriver(coordination.CoordinationDriver,
                 with open(group_meta_path, "wb") as fh:
                     fh.write(details_blob)
 
-        fut = self._submit(_do_create_group)
+        fut = self._executor.submit(_do_create_group)
         return FileFutureResult(fut)
 
     def join_group(self, group_id, capabilities=b""):
@@ -222,7 +290,7 @@ class FileDriver(coordination.CoordinationDriver,
                 fh.write(details_blob)
             self._joined_groups.add(group_id)
 
-        fut = self._submit(_do_join_group)
+        fut = self._executor.submit(_do_join_group)
         return FileFutureResult(fut)
 
     def leave_group(self, group_id):
@@ -245,7 +313,7 @@ class FileDriver(coordination.CoordinationDriver,
             else:
                 self._joined_groups.discard(group_id)
 
-        fut = self._submit(_do_leave_group)
+        fut = self._executor.submit(_do_leave_group)
         return FileFutureResult(fut)
 
     def get_members(self, group_id):
@@ -255,13 +323,8 @@ class FileDriver(coordination.CoordinationDriver,
         def _read_member_id(path):
             with open(path, 'rb') as fh:
                 contents = fh.read()
-                details = utils.loads(contents)
-                if isinstance(details, (dict)):
-                    return details['member_id']
-                else:
-                    raise TypeError(
-                        "Expected dict encoded in '%s'"
-                        " but got %s instead" % (path, type(details)))
+                details = _validate(utils.loads(contents), 'member')
+                return details['member_id']
 
         @_lock_me(self._driver_lock)
         def _do_get_members():
@@ -276,7 +339,7 @@ class FileDriver(coordination.CoordinationDriver,
                     raise
             else:
                 for entry in entries:
-                    if entry == ".metadata":
+                    if not entry.endswith('.raw'):
                         continue
                     entry_path = os.path.join(group_dir, entry)
                     try:
@@ -288,7 +351,7 @@ class FileDriver(coordination.CoordinationDriver,
                         members.append(member_id)
             return members
 
-        fut = self._submit(_do_get_members)
+        fut = self._executor.submit(_do_get_members)
         return FileFutureResult(fut)
 
     def get_member_capabilities(self, group_id, member_id):
@@ -312,14 +375,10 @@ class FileDriver(coordination.CoordinationDriver,
                 else:
                     raise
             else:
-                details = utils.loads(contents)
-                if not isinstance(details, (dict)):
-                    raise TypeError("Expected dict encoded in '%s'"
-                                    " but got %s instead" % (member_path,
-                                                             type(details)))
-                return details["capabilities"]
+                details = _validate(utils.loads(contents), 'member')
+                return details.get("capabilities")
 
-        fut = self._submit(_do_get_member_capabilities)
+        fut = self._executor.submit(_do_get_member_capabilities)
         return FileFutureResult(fut)
 
     def delete_group(self, group_id):
@@ -345,7 +404,7 @@ class FileDriver(coordination.CoordinationDriver,
                         if e.errno != errno.ENOENT:
                             raise
 
-        fut = self._submit(_do_delete_group)
+        fut = self._executor.submit(_do_delete_group)
         return FileFutureResult(fut)
 
     def get_groups(self):
@@ -353,11 +412,7 @@ class FileDriver(coordination.CoordinationDriver,
         def _read_group_id(path):
             with open(path, 'rb') as fh:
                 contents = fh.read()
-                details = utils.loads(contents)
-                if not isinstance(details, (dict)):
-                    raise TypeError("Expected dict encoded in '%s'"
-                                    " but got %s instead" % (path,
-                                                             type(details)))
+                details = _validate(utils.loads(contents), 'group')
                 return details['group_id']
 
         @_lock_me(self._driver_lock)
@@ -372,7 +427,7 @@ class FileDriver(coordination.CoordinationDriver,
                         raise
             return groups
 
-        fut = self._submit(_do_get_groups)
+        fut = self._executor.submit(_do_get_groups)
         return FileFutureResult(fut)
 
     def _init_watch_group(self, group_id):
