@@ -29,10 +29,12 @@ LOG = logging.getLogger(__name__)
 class MySQLLock(locking.Lock):
     """A MySQL based lock."""
 
+    MYSQL_DEFAULT_PORT = 3306
+
     def __init__(self, name, parsed_url, options):
         super(MySQLLock, self).__init__(name)
-        self._conn = MySQLDriver.get_connection(parsed_url, options)
         self.acquired = False
+        self._conn = MySQLDriver.get_connection(parsed_url, options)
 
     def acquire(self, blocking=True):
 
@@ -71,11 +73,14 @@ class MySQLLock(locking.Lock):
         return _lock()
 
     def release(self):
+        if not self.acquired:
+            return False
         try:
             with self._conn as cur:
                 cur.execute("SELECT RELEASE_LOCK(%s);", self.name)
                 cur.fetchone()
                 self.acquired = False
+                return True
         except pymysql.MySQLError as e:
             coordination.raise_with_cause(coordination.ToozError,
                                           utils.exception_message(e),
@@ -101,7 +106,7 @@ class MySQLDriver(coordination.CoordinationDriver):
         """Initialize the MySQL driver."""
         super(MySQLDriver, self).__init__()
         self._parsed_url = parsed_url
-        self._options = options
+        self._options = utils.collapse(options)
 
     def _start(self):
         self._conn = MySQLDriver.get_connection(self._parsed_url,
@@ -140,11 +145,11 @@ class MySQLDriver(coordination.CoordinationDriver):
     @staticmethod
     def get_connection(parsed_url, options):
         host = parsed_url.hostname
-        port = parsed_url.port
+        port = parsed_url.port or MySQLLock.MYSQL_DEFAULT_PORT
         dbname = parsed_url.path[1:]
         username = parsed_url.username
         password = parsed_url.password
-        unix_socket = options.get("unix_socket", [None])[-1]
+        unix_socket = options.get("unix_socket")
 
         try:
             if unix_socket:
