@@ -93,8 +93,8 @@ class PostgresLock(locking.Lock):
 
     def __init__(self, name, parsed_url, options):
         super(PostgresLock, self).__init__(name)
-        self._conn = PostgresDriver.get_connection(parsed_url, options)
         self.acquired = False
+        self._conn = PostgresDriver.get_connection(parsed_url, options)
         h = hashlib.md5()
         h.update(name)
         if six.PY2:
@@ -103,6 +103,7 @@ class PostgresLock(locking.Lock):
             self.key = h.digest()[0:2]
 
     def acquire(self, blocking=True):
+
         @_retry.retry(stop_max_delay=blocking)
         def _lock():
             # NOTE(sileht) One the same session the lock is not exclusive
@@ -129,16 +130,17 @@ class PostgresLock(locking.Lock):
                     else:
                         raise _retry.Retry
 
-        kwargs = _retry.RETRYING_KWARGS.copy()
-        kwargs['stop_max_delay'] = blocking
         return _lock()
 
     def release(self):
+        if not self.acquired:
+            return False
         with _translating_cursor(self._conn) as cur:
             cur.execute("SELECT pg_advisory_unlock(%s, %s);",
                         self.key)
             cur.fetchone()
             self.acquired = False
+            return True
 
     def __del__(self):
         if self.acquired:
@@ -160,7 +162,7 @@ class PostgresDriver(coordination.CoordinationDriver):
         """Initialize the PostgreSQL driver."""
         super(PostgresDriver, self).__init__()
         self._parsed_url = parsed_url
-        self._options = options
+        self._options = utils.collapse(options)
 
     def _start(self):
         self._conn = PostgresDriver.get_connection(self._parsed_url,
@@ -198,9 +200,9 @@ class PostgresDriver(coordination.CoordinationDriver):
 
     @staticmethod
     def get_connection(parsed_url, options):
-        host = options.get("host", [None])[-1]
-        port = parsed_url.port or options.get("port", [None])[-1]
-        dbname = parsed_url.path[1:] or options.get("dbname", [None])[-1]
+        host = options.get("host")
+        port = parsed_url.port or options.get("port")
+        dbname = parsed_url.path[1:] or options.get("dbname")
         username = parsed_url.username
         password = parsed_url.password
 
