@@ -62,17 +62,22 @@ class RedisLock(locking.Lock):
                                          timeout=timeout,
                                          thread_local=False)
         self._coord = coord
+        self._client = client
         self.acquired = False
 
+    def is_still_owner(self):
+        with _translate_failures():
+            lock_tok = self._lock.local.token
+            if not lock_tok:
+                return False
+            owner_tok = self._client.get(self.name)
+            return owner_tok == lock_tok
+
     def acquire(self, blocking=True):
-        if blocking is True or blocking is False:
-            blocking_timeout = None
-        else:
-            blocking_timeout = float(blocking)
-            blocking = True
+        blocking, timeout = utils.convert_blocking(blocking)
         with _translate_failures():
             self.acquired = self._lock.acquire(
-                blocking=blocking, blocking_timeout=blocking_timeout)
+                blocking=blocking, blocking_timeout=timeout)
             if self.acquired:
                 self._coord._acquired_locks.add(self)
             return self.acquired
@@ -142,6 +147,13 @@ class RedisDriver(coordination._RunWatchersMixin,
     that gets worked on). See http://redis.io/topics/transactions for more
     information on this topic.
 
+    General recommendations/usage considerations:
+
+    - When used for locks, run in AOF mode and think carefully about how
+      your redis deployment handles losing a server (the clustering support
+      is supposed to aid in losing servers, but it is also of unknown
+      reliablity and is relatively new, so use at your own risk).
+
     .. _redis: http://redis.io/
     .. _msgpack: http://msgpack.org/
     .. _sentinel: http://redis.io/topics/sentinel
