@@ -104,6 +104,15 @@ def safe_abs_path(rooted_at, *pieces):
     return path
 
 
+def convert_blocking(blocking):
+    """Converts a multi-type blocking variable into its derivatives."""
+    timeout = None
+    if not isinstance(blocking, bool):
+        timeout = float(blocking)
+        blocking = True
+    return blocking, timeout
+
+
 def ensure_tree(path):
     """Create a directory (and any ancestor directories required).
 
